# Filters added to this controller apply to all controllers in the application.
# Likewise, all the methods added will be available for all controllers.

class ApplicationController < ActionController::Base
  before_filter :store_location

  def set_locale
  	I18n.locale = (current_user.locale if current_user) || I18n.default_locale
  end
  helper :all # include all helpers, all the time
  protect_from_forgery # See ActionController::RequestForgeryProtection for details

  # Scrub sensitive parameters from your log
  filter_parameter_logging :password, :password_confirmation
  helper_method :current_user_session, :current_user, :facebook_user
  
	def absolute_url(path)
    request.protocol + request.host_with_port + path
  end
    
  
  private

  def update_facebook_permissions
    @current_user = current_user || UserSession.find.user rescue false
    if @current_user
      @current_user.check_perm_publish_stream
    end
  end

  def current_user_session
    return @current_user_session if defined?(@current_user_session)
    @current_user_session = UserSession.find
  end

  def current_user
    return @current_user if defined?(@current_user)
    @current_user = current_user_session && current_user_session.user
    facebook_user
    @current_user
  end
  
  def facebook_user
    return @facebook_user if defined?(@facebook_user)
    begin
      if session[:facebook_session] && session[:facebook_session].user && session[:facebook_session].user.name
        if (current_user && current_user.facebook_uid != session[:facebook_session].user.uid)
          return false
        else
          @facebook_user = session[:facebook_session].user
        end
      end
    rescue Facebooker::Session::SessionExpired
      flash.now[:info] = "Sesja Facebook'a wygasła"
      clear_facebook_session_information
      clear_fb_cookies!
    end
    
  end
  
  def require_user
    unless current_user
      restric_access
      return false
    end
  end
  
  def require_not_logged_in
    if @current_user || current_user
      redirect_to root_url
    end
  end

  def require_special_user
    unless current_user and (current_user.is_specialuser? or current_user.is_superadmin?)
      restric_access
      return false
    end
  end

  def require_admin
    unless current_user and current_user.is_superadmin?
      restric_access
      return false
    end
  end

  def restric_access
    # Dwie opcje do wyboru
    # 1. pokazać stronę z blędem 401
    # 2. ustawić flasha i przekierować do logowania

    #render :file => "#{RAILS_ROOT}/public/401.html", :status => 401
    flash[:info] = "Nie masz dostępu do tej strony"
    respond_to do |f|
      f.html {redirect_to new_user_session_url}
      f.js do
        render "layouts/blank"
      end
    end
  end

  def store_location
    session[:return_get] = request.get?
    session[:return_to] = request.get? ? request.request_uri : request.env["HTTP_REFERER"]
  end

  def redirect_back_or_default(default)
    redirect_to(session[:return_to] || default)
    session[:return_to] = nil
  end

end

