class FacebookController < ApplicationController
  before_filter :set_facebook_session, :only => :connect
  before_filter :require_not_logged_in, :only => [:create]
  before_filter :require_facebook_user, :only => [:new, :create, :connect, :link, :login_and_link]
  before_filter :require_user, :only => [:link, :unlink, :fill_missing, :update_missing, :check_permissions]
  #after_filter :request_additional_permissions, :only => [:create, :link]


  def connect
    #znajdz usera z danym facebook_uid
    @user = User.find_by_facebook_uid(@facebook_user.uid) 
    
    #jesli wczesniej byl zalogowany natywnie, sprawdz czy ma podpiete to samo konto FB
    if @current_user && @user && @current_user != @user
      @facebook_user = nil
      flash[:info] = "To konto Facebook'a jest połączone z innym kontem na Barcamp.pl. Możesz je rozłączyć w ustawieniach konta '#{@user.login}'."
      redirect_to root_url
      
    #jesli znalazl i jest ok, zaloguj  
    elsif @user 
      @user_session = UserSession.create(@user)
      update_facebook_permissions
      flash[:notice] = t("user.logged_in")
      redirect_to root_url
      
    #jesli nie znalazl, poszukaj po mailu
    elsif @user_by_mail = User.find_by_email(@facebook_user.email) 
      @user_session = UserSession.create(@user_by_mail)
      flash[:notice] = "Połączono z kontem #{@user_by_mail.login} na podstawie adresu email."
      redirect_to link_facebook_path
    
    #jesli wczesniej natywnie zalogowany, linkuj (wewnatrz link: tylko jesli current_user.facebook_uid == nil)  
    elsif @current_user
      redirect_to link_facebook_path

    #nowy user
    else
      redirect_to new_facebook_path
    end
  end
  
  def new
    @user = User.new
    @user.login = @facebook_user.name.gsub(" ", "_").downcase
    @user_session = UserSession.new
    #@user_session.authenticating_with_facebook_connect?
  end
  
  def create
    @user = User.new(params[:user])
    @user.superadmin = false
    @user.facebook_name = @facebook_user.name
    @user.facebook_uid = @facebook_user.uid
    @user.email = @facebook_user.email
    @user.password = @user.password_confirmation = Authlogic::Random::friendly_token
    @user.confirmed = true
    @user.facebook_only = true
    
    if @user.save
      @user_session = UserSession.create(@user)
      flash[:notice] = "Zalogowano."
      redirect_to root_url
    else
      @user_session = UserSession.new
      render :action => :new, :controller => :facebook
    end
  end
  
  def login_and_link
    @user_session = UserSession.new(params[:user_session])
    if @user_session.save
      redirect_to :action  => :link, :controller => :facebook
    else
      flash[:error] = "Nieprawidłowy login lub hasło."
      redirect_to new_facebook_path
    end
  end
  
  
  def link
    @user = current_user
    unless @user.facebook_uid
      @user.facebook_name = @facebook_user.name
      @user.facebook_uid = @facebook_user.uid
      @user.facebook_only = false
      @user.save
      if flash[:notice] 
        flash.keep # bo jesli jest linkowany na podstawie maila to jest juz flash z facebook/connect
      else
        flash[:notice] = "Pomyślnie połączono z kontem #{@user.facebook_name}" if @user.save
      end
    else
      flash[:info] = "Konto #{@user.login} jest już połączone z Facebookiem."
    end
    redirect_to account_path
  end
  
  def check_permissions
    @user = current_user
    if @user.facebook_uid && @facebook_user
      #@result = @facebook_user.has_permissions?(["email","publish_stream","offline_access"])
      @result = @facebook_user.has_permission?("publish_stream")
      @user.update_attribute(:facebook_perm_publish_stream, @result)
      if @user.facebook_proxy_email? && !@facebook_user.email.include?("@proxymail.facebook.com") && !@facebook_user.email.blank?
        @user.update_attribute(:email, @facebook_user.email)
      end
      if @result
        flash[:notice] = "Rozszerzone uprawnienia są włączone."
      else
        flash[:info] = "Brak uprawnień do automatycznej publikacji."
      end
    end
    
    respond_to do |f|
      f.html {redirect_to :controller => :accounts, :action => :show}
      f.js do
        render "layouts/blank"
      end
    end
  end
  
  def fill_missing
    @user = current_user
    @user.email = nil if @user.facebook_proxy_email?
  end
  
  def missing_update
    @user = current_user
    if @user.facebook_only?
      @user.facebook_only = false
      if @user.update_attributes(params[:user])
        flash[:notice] = "Zaktualizowano pomyślnie."
        redirect_to account_path
      else
        flash[:error] = "Nieprawidłowe dane."
        redirect_to :action => :fill_missing, :controller => :facebook
      end
    end
  end
  
  def unlink
    unless current_user.facebook_only?
      current_user.facebook_uid = nil
      current_user.facebook_name = nil
      current_user.facebook_session_key = nil
      session[:facebook_session] = nil
      @facebook_user = nil
      flash[:notice] = "Konto odłączono pomyślnie." if current_user.save
    end
    redirect_to account_path
  end
  
  
  private
  
  #to samo co facebook_user w app_controller tylko nie sprawdza zgonosci uid
  def require_facebook_user
    begin
      if session[:facebook_session] && session[:facebook_session].user && session[:facebook_session].user.name
        @facebook_user = session[:facebook_session].user
      else
        flash[:error] = "Brak sesji Facebook'a."
        redirect_to root_url
      end
    rescue Facebooker::Session::SessionExpired
      flash.now[:info] = "Sesja Facebook'a wygasła."
      clear_facebook_session_information
      clear_fb_cookies!
      redirect_to root_url
    end
  end
  
  def request_additional_permissions
    flash[:fb_action] = "FB.Connect.showPermissionDialog('publish_stream,email', fb_permission_handler);"
  end
end