class User < ActiveRecord::Base
  
  acts_as_authentic
  
  has_many :speakers, :dependent => :nullify
  has_many :event_groups, :dependent => :nullify
  has_many :comments, :dependent => :nullify
  has_many :attendances
  has_many :events, :through => :attendances, :dependent => :destroy

  attr_accessible :login, :password, :password_confirmation, :email, :locale, :events_remind, :events_advertise, :facebook_publish

  # TODO zamienic na attr_accessible

  validates_format_of :login, :with => /^[a-zA-Z0-9_-]+$/, :message => "powinien składać się tylko z liter, liczb i znaków -_"

  named_scope :events_advertise, :conditions => {:events_advertise => true}
  named_scope :events_remind, :conditions => {:events_remind => true}
  
  def facebook_only?
    facebook_only
  end
  
  def facebook_proxy_email?
    email.include?("@proxymail.facebook.com")
  end
  
  def create_facebook_session #dla uzycia wewnatrz modelu api FB
    if facebook_uid
      @fb_session = Facebooker::Session.create
      @fb_user = Facebooker::User.new(facebook_uid, @fb_session)
    else
      false
    end
  end
  
  def check_facebook_permission(permission)
    if facebook_uid && create_facebook_session
      @fb_user.has_permission?(permission)
    else
      false
    end
  end
  
  def check_perm_publish_stream
    begin
      if self.facebook_publish
        self.facebook_perm_publish_stream = check_facebook_permission("publish_stream")
        save
        facebook_perm_publish_stream
      end
    rescue
    end
  end

  def is_superadmin?
    self.superadmin
  end

  def is_specialuser?
    !self.event_groups.blank? or self.superadmin
  end

  def is_event_group_admin?(event_group)
    self.event_groups.exists?(event_group) or self.superadmin
  end

  def deliver_password_reset_instructions!
    reset_perishable_token!
    Mailer.deliver_password_reset_instructions(self)
  end

  def deliver_confirm_instructions!
    reset_perishable_token!
    Mailer.deliver_user_confirm_instructions(self)
  end
  
  def to_s
    self.login
  end

  def confirm!
    self.confirmed = true
    save
  end

  def confirmed?
    self.confirmed
  end
end

